@@ -9,7 +9,7 @@ from pydantic import BaseModel
 
 import fps
 from fps.errors import ConfigError
-from fps.utils import get_plugin_name
+from fps.utils import get_pkg_name, get_plugin_name
 
 
 class PluginModel(BaseModel):
@@ -37,29 +37,28 @@ logger = logging.getLogger("fps")
 
 class Config:
 
-    _models: Dict[Type[PluginModel], Tuple[ModuleType, PluginModel]] = {}
+    _models: Dict[Type[PluginModel], Tuple[str, PluginModel]] = {}
     _based_on: Dict[Type[PluginModel], List[str]] = {}
-    _obj: dict = None
     _files: Dict[str, dict] = {}
-    _names: dict = {}
+    _plugin2name: Dict[ModuleType, str] = {}
+    _pkg2name: Dict[str, str] = {}
 
     def __new__(cls, plugin_model: Type[PluginModel], file: str = None) -> PluginModel:
         try:
             return cls._models[plugin_model][1]
         except KeyError:
-            logger.error(f"Plugin config model {plugin_model} is not yet registered")
+            logger.error(f"Config model '{plugin_model}' is not yet registered")
             exit(1)
 
     @classmethod
     def register(
         cls,
-        plugin: ModuleType,
-        plugin_model: Type[PluginModel],
+        config_name: ModuleType,
+        config_model: Type[PluginModel],
         force_update: bool = False,
     ):
         config_objs: List[dict] = []
-        plugin_name = cls.plugin_name(plugin)
-        files = cls.find_files(plugin_name)
+        files = cls.find_files(config_name)
 
         # parse files is not yet done
         for f in files:
@@ -74,29 +73,29 @@ class Config:
             relevant_files = [
                 f
                 for i, f in enumerate(files)
-                if plugin_name in cls._files[f] and cls._files[f][plugin_name]
+                if config_name in cls._files[f] and cls._files[f][config_name]
             ]
             config_obj = {
                 k: v
                 for f in relevant_files
-                for k, v in cls._files[f][plugin_name].items()
+                for k, v in cls._files[f][config_name].items()
             }
         else:
             relevant_files = []
             config_obj = {}
 
         # update the config if new files are detected
-        update = cls._based_on.get(plugin_model, []) != relevant_files
+        update = cls._based_on.get(config_model, []) != relevant_files
 
         # compute/update config if necessary
-        if plugin_model not in cls._models or update or force_update:
+        if config_model not in cls._models or update or force_update:
             if config_obj:
-                config = plugin_model.parse_obj(config_obj)
+                config = config_model.parse_obj(config_obj)
             else:
-                config = plugin_model()
+                config = config_model()
 
-            cls._models[plugin_model] = (plugin, config)
-            cls._based_on[plugin_model] = relevant_files
+            cls._models[config_model] = (config_name, config)
+            cls._based_on[config_model] = relevant_files
 
     @classmethod
     def update(cls, force: bool = False):
@@ -114,29 +113,58 @@ class Config:
                 raise ConfigError(f"Failed to load config file '{path}': {e}.")
 
     @staticmethod
-    def find_files(plugin_name: str):
+    def find_files(config_name: str):
         ext = ".toml"
-        config_files = [f + ext for f in ("fps", plugin_name)]
+        config_files = [f + ext for f in ("fps", config_name)]
 
+        # file define by user in env variable 'FPS_CONFIG_FILE'
         if "FPS_CONFIG_FILE" in os.environ:
             config_files.append(os.environ["FPS_CONFIG_FILE"])
 
+        # file passed by user as a CLI argument '--config'
+        if "FPS_EXTRA_CONFIG_FILE" in os.environ:
+            config_files.append(os.environ["FPS_EXTRA_CONFIG_FILE"])
+
+        # file generated by the CLI from extra args
+        if "FPS_CLI_CONFIG_FILE" in os.environ:
+            config_files.append(os.environ["FPS_CLI_CONFIG_FILE"])
+
         config_files = list(OrderedDict.fromkeys(config_files))
 
         return [f for f in config_files if f and os.path.isfile(f)]
 
     @classmethod
-    def register_plugin_name(cls, plugin, name: str):
-        cls._names[plugin] = name
+    def register_plugin_name(cls, plugin: ModuleType, name: str = None):
+        plugin_name = get_plugin_name(plugin)
+        pkg = get_pkg_name(plugin, strip_fps=False)
+
+        if not name:
+            pkg_name = get_pkg_name(plugin, strip_fps=False)
+            if pkg_name in cls._pkg2name:
+                name = cls._pkg2name[pkg_name]
+                logger.debug(f"Re-using plugins package name '{name}'")
+            else:
+                name = get_pkg_name(plugin, strip_fps=True)
+                logger.debug(f"Using default name '{name}' for plugins package '{pkg}'")
+
+        if pkg not in cls._pkg2name:
+            logger.info(f"Registering name '{name}' for plugins package '{pkg}'")
+
+        logger.debug(f"Registering name '{name}' for plugins '{plugin_name}'")
+
+        cls._plugin2name[plugin] = name
+        cls._pkg2name[pkg] = name
+
+        return name
 
     @classmethod
     def plugin_name(cls, plugin: ModuleType) -> str:
-        if plugin in cls._names:
-            return cls._names[plugin]
-        else:
-            return get_plugin_name(plugin)
+        try:
+            return cls._plugin2name[plugin]
+        except KeyError:
+            return cls.register_plugin_name(plugin)
 
     @classmethod
-    @property
-    def plugins_names(cls) -> List[str]:
-        return list(cls._names.values())
+    def clear_names(cls):
+        cls._plugin2name.clear()
+        cls._pkg2name.clear()
