@@ -1,3 +1,4 @@
+import http
 import logging
 import logging.config
 import os
@@ -78,6 +79,53 @@ class ColourizedFormatter(logging.Formatter):
         return super().formatMessage(record)
 
 
+class AccessFormatter(ColourizedFormatter):
+    status_code_colours = {
+        1: lambda code: click.style(str(code), fg="bright_white", bold=True),
+        2: lambda code: click.style(str(code), fg="green", bold=True),
+        3: lambda code: click.style(str(code), fg="yellow", bold=True),
+        4: lambda code: click.style(str(code), fg="red", bold=True),
+        5: lambda code: click.style(str(code), fg="bright_red", bold=True),
+    }
+
+    def get_status_code(self, status_code: int) -> str:
+        try:
+            status_phrase = http.HTTPStatus(status_code).phrase
+        except ValueError:
+            status_phrase = ""
+        status_and_phrase = "%s %s" % (status_code, status_phrase)
+        if self.use_colors:
+
+            def default(code: int) -> str:
+                return status_and_phrase  # pragma: no cover
+
+            func = self.status_code_colours.get(status_code // 100, default)
+            return func(status_and_phrase)
+        return status_and_phrase
+
+    def formatMessage(self, record: logging.LogRecord) -> str:
+        recordcopy = copy(record)
+        (
+            client_addr,
+            method,
+            full_path,
+            http_version,
+            status_code,
+        ) = recordcopy.args
+        status_code = self.get_status_code(int(status_code))
+        request_line = "%s %s HTTP/%s" % (method, full_path, http_version)
+        if self.use_colors:
+            request_line = click.style(request_line, bold=False)
+        recordcopy.__dict__.update(
+            {
+                "client_addr": client_addr,
+                "request_line": request_line,
+                "status_code": status_code,
+            }
+        )
+        return super().formatMessage(recordcopy)
+
+
 def colourized_formatter(
     fmt: Optional[str] = "",
     datefmt: Optional[str] = None,
@@ -96,8 +144,10 @@ def merge_configs(a, b, path=None):
         if key in a:
             if isinstance(a[key], dict) and isinstance(b[key], dict):
                 merge_configs(a[key], b[key], path + [str(key)])
+                continue
+
             if isinstance(a[key], list) and isinstance(b[key], list):
-                a[key] = list(tuple(a[key] + b[key]))
+                a[key] = list(set(a[key] + b[key]))
             elif a[key] == b[key]:
                 pass  # same leaf value
             else:
@@ -127,6 +177,13 @@ def get_logger_config(loggers=()):
             "format": "[%(levelname).1s %(asctime)s %(name)s] %(message)s",
             "datefmt": "%Y-%m-%d %H:%M:%S",
         },
+        "access": {
+            "()": "fps.logging.AccessFormatter",
+            "fmt": '%^[%(levelname)s %(asctime)s %(name)s]%$ %(client_addr)s - "'
+            '%(request_line)s" %(status_code)s',
+            "datefmt": "%Y-%m-%d %H:%M:%S",
+            "use_colors": True,
+        },
     }
 
     LOG_HANDLERS = {
@@ -136,6 +193,12 @@ def get_logger_config(loggers=()):
             "level": log_level,
             "stream": "ext://sys.stderr",
         },
+        "console_access": {
+            "class": "logging.StreamHandler",
+            "formatter": "access",
+            "level": log_level,
+            "stream": "ext://sys.stderr",
+        },
         "file": {
             "class": "logging.FileHandler",
             "formatter": "timestamp",
@@ -159,6 +222,8 @@ def get_logger_config(loggers=()):
             "loggers": LOGGERS,
         },
     )
+    if "uvicorn.access" in LOG_CONFIG["loggers"]:
+        LOG_CONFIG["loggers"]["uvicorn.access"]["handlers"] = ["console_access"]
 
     return LOG_CONFIG
 
@@ -167,3 +232,4 @@ def configure_logger(loggers):
     """Get quetz logger"""
     log_config = get_logger_config(loggers)
     logging.config.dictConfig(log_config)
+    return log_config
